/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file bldc_isr.c
*
* @brief This file provides BLDC control interruptcallback functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "ac780x_pwdt_reg.h"
#include "bldc_keys.h"
#include "bldc_isr.h"
#include "bldc_pwm.h"
#include "bldc_adc.h"
#include "bldc_bemf.h"
#include "task_scheduler.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Callback routine of TIMER1, 1ms. The Task scheduler update in Timer1 interrupt,use for bldc control time base. 
*
* @param[in] device: device
* @param[in] wparam: callback parameter
* @param[in] lpara: callback parameter
* @return none
*/
void BLDC_Base_TimerCallback(void *device, uint32_t wpara, uint32_t lpara)
{
    /* same as FOC_TIMER callback, to update the time base tasks in TIMER_ISR */
    if (wpara & TIMER_CHANNEL_TF_TFLG_Msk)
    {
        SCH_Update();
    }
}

/*!
* @brief Callback routine of TIMER2, use for back EMF zero crossing detection in bldc sensorless control. 
*
* @param[in] device: device
* @param[in] wparam: callback parameter
* @param[in] lpara: callback parameter
* @return none
*/
void Timer2_Callback(void *device, uint32_t wpara, uint32_t lpara)
{
    if (wpara & TIMER_CHANNEL_TF_TFLG_Msk)
    {
        if (g_mcStatus == RUN)
        {
            /* 60000 * 6 * TIMER_XUS = 8.64s, constrained the minmum operating elec frequency 0.12Hz */
            if (g_bemfControl.changePhaseTime < 60000)
            {
                g_bemfControl.changePhaseTime++;
            }
            BemfCheckFun(&g_bldcVarsCtrl, pBldcVarsCfg);
        }
    }
}

/*!
* @brief Callback routine of ADC, use for ADC sample data and current loop in bldc control.
*
* @param[in] device: device
* @param[in] wparam: callback parameter
* @param[in] lpara: callback parameter
* @return none
*/
void BLDC_ADCCallback(void *device, uint32_t wpara, uint32_t lpara)
{
    if (wpara & ADC_STR_IEOC_Msk)
    {
    #if (defined DAC_FUNCTIONALITY)
        PWM_SetChannelCountValue(PWM0, PWM_CH_0, ((g_bldc_speedCmd.speedTargetRamp + 32768) >> 4));
        PWM_SetChannelCountValue(PWM0, PWM_CH_7, ((g_bldc_speedCmd.speedFbk + 32768) >> 4));
    #endif
        Get_VoltageValue(&g_bldc_adSample);
        if ((g_mcStatus == FAULT) || (g_mcStatus == STOP) || (g_mcStatus == IDLE) || (g_mcStatus == POWER_ON))
        {
            BLDC_Stop();
            return;
        }
        else if (g_mcStatus == RUN)
        {
            BusCurrent_Control(&g_bldcVarsCtrl, &g_bldc_adSample);
        }
        else
        {

        }
    }
}

/*!
* @brief Callback routine of PWM, occur while hardware over current trigger PWM fault, use for hardware over current protection in bldc control.
*
* @param[in] device: device
* @param[in] wparam: callback parameter
* @param[in] lpara: callback parameter
* @return none
*/
void BLDC_PWM_FaultCallback(void *device, uint32_t wpara, uint32_t lpara)
{
    if (PWM_GetOverflowFlag(PWM1))
    {
//        PWM_ClrOverFlowFlag(PWM1);
    }
    if (PWM_GetFaultChannelFlag(PWM1, (uint16_t)PWM_FAULT_CH_1))
    {
        g_mcStatus = FAULT;
        g_bldc_protector.faultFlag.bits.hwFault = 1;
        PWM_ClearFaultChannelFlag(PWM1, (uint16_t)PWM_FAULT_CH_1);
    }
}

/*!
* @brief Callback routine of PWDT, occur while hall signal update, use for back EMF zero crossing detection hall signal captrue in bldc sensorless control.
*
* @param[in] device: device
* @param[in] wparam: callback parameter
* @param[in] lpara: callback parameter
* @return none
*/
void BLDC_Bemf_PWDTCallback(void *device, uint32_t wpara, uint32_t lpara)
{
    if (wpara & PWDT_INIT0_OVF_Msk)
    {

    }

    if (wpara & PWDT_INIT0_RDYF_Msk)
    {

    }
}

/*!
* @brief Callback routine of PWDT, occur while hall signal update, use for hall signal captrue in bldc control with hall sensor.
*
* @param[in] device: device
* @param[in] wparam: callback parameter
* @param[in] lpara: callback parameter
* @return none
*/
void BLDC_Hall_PWDTCallback(void *device, uint32_t wpara, uint32_t lpara)
{
    uint32_t temp_PwdtNpw;
    uint32_t temp_PwdtInit0;
    temp_PwdtNpw = PWDT0->NPW;
    temp_PwdtInit0 = PWDT0->INIT0;

    if (wpara & PWDT_INIT0_OVF_Msk)
    {
        memset(&g_hallControl.totalPulseWidth, 0U , sizeof(g_hallControl.totalPulseWidth));
        if (0 == g_bldc_protector.stallFlag)
        {
            g_hallControl.pwdtValid = 0;
        }
        else
        {
            g_hallControl.currentPhase = PWDT_GetHallStatus(PWDT0) & 0x07;
            if (g_mcStatus == RUN)
            {
                BLDC_Commutate_BaseSector(pBldcVarsCfg->motorDir, g_hallControl.currentPhase);
            }
        }
        return;
    }

    if (wpara & PWDT_INIT0_RDYF_Msk)
    {
        if (g_hallControl.pwdtValid < 2)
        {
            g_hallControl.pwdtValid++;
            return;
        }
        g_hallControl.currentPhase = PWDT_GetHallStatus(PWDT0) & 0x07;
        StorageDataCyclic(&g_hallControl.totalPulseWidth, ((temp_PwdtNpw & 0xFFFF) + ((temp_PwdtInit0 >> 16) & 0xFFFF)));
        if (g_mcStatus == RUN)
        {
            BLDC_Commutate_BaseSector(pBldcVarsCfg->motorDir, g_hallControl.currentPhase);
        }
    }
}

/* =============================================  EOF  ============================================== */
