/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __HALL_SENSOR_H__
#define __HALL_SENSOR_H__
/*!
* @file hall_sensor.h
*
* @brief  This file provides hall eletric theta  and hall self-learn functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include "ac780x_pwdt_reg.h"
#include "param_init.h"
#include "pid_regulator.h"

/* ============================================  Define  ============================================ */
#define ZERO_ORDER_FORCE_COMP           (0)                             /*!<  define force degree compensation mode */
#define ZERO_ORDER_DISPERSE_COMP        (1)                             /*!< define dispere degree compensation mode */

#define PHASE_SHIFT_05                  (PHASE_SHIFT_1 >> 1)            /*!< define 0.5 degree in Q16 format */
#define PHASE_SHIFT_1                   (uint16_t)(65536 * 1 / 360)     /*!< define 1 degree in Q16 format */
#define PHASE_SHIFT_2                   (uint16_t)(65536 * 2 / 360)     /*!< define 2 degree in Q16 format */
#define PHASE_SHIFT_5                   (uint16_t)(65536 * 5 / 360)     /*!< define 5 degree in Q16 format */
#define PHASE_SHIFT_10                  (uint16_t)(65536 * 10 / 360)    /*!< define 10 degree in Q16 format */
#define PHASE_SHIFT_30                  (uint16_t)(65536 * 30 / 360)    /*!< define 30 degree in Q16 format */
#define PHASE_SHIFT_60                  (uint16_t)(65536 * 60 / 360)    /*!< define 60 degree in Q16 format */
#define PHASE_SHIFT_90                  (uint16_t)(65536 * 90 / 360)    /*!< define 90 degree in Q16 format */

#define CLOCKWISE                       (1)                             /*!< define motor run at clockwise direction */
#define ANTI_CLOCKWISE                  (-1)                            /*!< define motor run at anti-clockwise direction */

#define SELF_LEARN_START                (0)                             /*!< define hall self-learn zero stage*/
#define SELF_LEARN_10DEGREE_FINISHED    (1)                             /*!< define hall self-learn first stage*/
#define SELF_LEARN_2DEGREE_FINISHED     (2)                             /*!< define hall self-learn second stage*/
#define SELF_LEARN_HALL_WIDTH           (3)                             /*!< define hall self-learn third stage*/

#define IF_RUN_FREQ                     (20ul)                          /*!<  define hall sector width self-learn IF run frequency*/

/* ===========================================  Typedef  ============================================ */
/*!
* @brief Hall sensor self learn data
*/
typedef struct
{
    uint8_t zeroPosHallStatus;      /*!< Hall Zero position */
    uint8_t selfLearnStage;         /*!< Hall learning stage */
    uint8_t selfLearnOrder;         /*!< Hall learning order */
    uint8_t selfLearnSubStage;      /*!< Hall learning sub-stage */
    uint8_t tenIntervalHallStatus;  /*!< Ten degree interval hall status */
    uint8_t twoIntervalHallStatus;  /*!< Two degree interval hall status */

    uint16_t selfLearnIndex;        /*!< Hall self learn data array index */
    uint16_t selfLearnTheta;        /*!< Hall learning theta */
    uint16_t selfLearnAngle;        /*!< Hall self learn angle */
    uint16_t tsCnt;                 /*!< Hall width learn counter */
    uint16_t freTarget;             /*!< Hall width learn target frequency */
    uint16_t sectorWidthTsMax;      /*!< Hall width learn ts max */
    uint16_t sectorWidthTsMin;      /*!< Hall width learn ts min */
    uint16_t freSet;                /*!< Hall width learn actual set frequency */
    uint16_t thetaDelta;            /*!< Hall width learn delta theta per ts */
    uint16_t t1msCnt;               /*!< Hall width learn 1ms counter */
    uint16_t widthLearnCnt;         /*!< Hall width learn counter */
    uint16_t setThetaAngle;         /*!< Hall sensor self-learning output angle */
    uint16_t sectorWidthLearnArray[6];  /*!< Hall sensor width learning value buffer */
    uint16_t sectorWidthMax;        /*!< Hall sensor sector width max value */
    uint16_t sectorWidthMin;        /*!< Hall sensor sector width min value */
} HALL_LEARN_DATA;

/*!
* @brief Hall sensor configuration
*/
typedef struct
{
    uint8_t hallSelfLearnMode;      /*!< Hall self learn mode, 1: learn order and theta; 2: learn order and theta and sector width */
    uint16_t hallCalcMode;          /*!< Hall calculate rotor position mode */
    uint16_t hallSelfLearnEnable;   /*!< Hall self learn enable */
    uint16_t hallSwapTrqCoff;       /*!< Hall torque boost cofficient at low speed */
    uint16_t learnIntervalTime;     /*!< Hall self learn interval time */
    uint16_t learnOrder;            /*!< Hall self learn finished get hall order */
    uint16_t learnDegree;           /*!< Hall self learn finished get hall install error */
    uint32_t learnCur;              /*!< Hall self learn current */
} HALL_SENSOR_CFG;

/*!
* @brief Hall sensor data
*/
typedef struct
{
    uint8_t sectorWidthIndex;       /*!< Sector width array index */
    uint8_t pwdtDatValid;           /*!< pwdt isr data valid flag */
    uint8_t hallEstThetaSectorNext; /*!< Hall est status next */
    uint8_t hallEstThetaSector;     /*!< Hall est status */
    uint8_t hallEstThetaSectorBuf;  /*!< Hall est status Buf */
    uint8_t swapstate;              /*!< 0: swap mode 1:unswap mode */

    uint16_t hallStatus;            /*!< Hall status */
    int16_t rotorDirection;         /*!< Motor rotor direction 1:clockwise -1:anti-clockwise */
    int16_t estThetaStep;           /*!< Estimation rotor step */
    int16_t estThetaErr;            /*!< Estimation rotor angle error */
    int16_t estFre;                 /*!< Estimation rotor frequency */
    uint16_t estTheta;              /*!< Estimation rotor angle,Q16 format */
    uint16_t estThetaBuf;           /*!< Estimation rotor angle buff,Q16 format */
    uint16_t estThetaCompCnt;       /*!< Decentralized theta compensation Ts counter */
    uint16_t estThetaCompCntBuf;    /*!< Decentralized theta compensation Ts counter buff */
    uint16_t maxEstTheta;           /*!< Max estimation theta */
    uint16_t minEstTheta;           /*!< Min estimation theta */

    uint32_t freCalCof;             /*!< Frequency calculate coff */
} HALL_SENSOR_DATA;

/*!
* @brief Hall sensor data assemble
*/
typedef struct
{
    HALL_SENSOR_CFG         *pHallCfg;                 /*!< Hall sensor configuration parameters */
    HALL_SENSOR_DATA        *pHallSensor;              /*!< Hall sensor calculate rotor position */
    HALL_LEARN_DATA         *pHallSelfLearn;           /*!< Hall sensor self learn data */
} HALL_SENSOR;

/* ==========================================  Variables  =========================================== */
/* Six sector boundary degree */
extern uint16_t g_hallSectorDegree[8];

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Hall sensor initialize function, this function initilize rotor position calculate data struct and hall install self-learn struct.
*        This function will be called at main status at STOP, and PWDT overflow interrupt.
*        If the motor have not self-learn, then this function will clear hall install imformation.
*        If the motor have been self-learn, the this function will use configuration imformation to calculate rotor position.
*
* @param[in] hallStatus: PWDT module output hall status
* @param[in] hall: HALL_SENSOR type pointer, pointer to g_hall variable
* @param[in] focVars: FOC_VARS type pointer, pointer to g_focVars, which contain FOC configure and FOC control
* @return none
*/
void HallSensor_Init(uint8_t hallStatus, HALL_SENSOR *hall, FOC_VARS *focVars);

/*!
* @brief Hall sensor PWDT interrupt service function, which is mainly calculate motor frequency.
*        When the three hall signals change, what ever rising edge or falling edge, will trriger PWDT modular generate interrupt.
*        Base on interrupts interval time, calculate motor frequency.
*
* @param[in] hallStatus: PWDT module output hall status
* @param[in] negSectorWidth: PWDT module output, negtive pulse width
* @param[in] posSectorWidth: PWDT module output, postive pulse width
* @param[in] hall: HALL_SENSOR type pointer, pointer to g_hall variable
* @param[in] focVarsCfg: FOC_VARS_CFG type pointer, pointer to defFocVars, which configure whole FOC control
* @return none
*/
void HallSensor_PwdtSubCallBack(uint8_t hallStatus, uint16_t negSectorWidth, uint16_t posSectorWidth, HALL_SENSOR *hall, FOC_VARS_CFG *focVarsCfg);

/*!
* @brief Hall sensor ADC interrupt service function, which is mainly calculate motor rotor position.
*        This function has two methods calculate rotor position, when motor run at low speed (swap status), the rotor position is each sector middle value.
*        When motor run at middle or high speed (unswap status), the rotor positioon is the frequency integal.
*
* @param[in] hallStatus: PWDT module output hall status
* @param[in] hallSensor: HALL_SENSOR_DATA type pointer, pointer to g_hallSensor variable
* @param[in] hallSensorCfg: HALL_SENSOR_CFG type pointer, pointer to defHallSensorCfg variable, which is hall mode configuration
* @return Calculated rotor position
*/
uint16_t HallSensor_AdcCallBack(uint8_t hallStatus, HALL_SENSOR_DATA *hallSensor, HALL_SENSOR_CFG *hallSensorCfg);

/*!
* @brief Hall sensor PWDT interrupt service function will call this function when motor run at low speed (swap status).
*        When enter PWDT interrupt, this function judge two interrupts interval time, and calculate frequency.
*        If the frequency reach the threshold value, then change sawp status to unsawp status.
*
* @param[in] negSectorWidth: PWDT module output, negtive pulse width
* @param[in] posSectorWidth: PWDT module output, postive pulse width
* @param[in] hallSensor: HALL_SENSOR_DATA type pointer, pointer to g_hallSensor variable
* @param[in] focVarsCfg: FOC_VARS_CFG type pointer, pointer to defFocVars, which configure whole FOC control
* @return none
*/
void HallSensor_SwapState(uint16_t negSectorWidth, uint16_t posSectorWidth, HALL_SENSOR_DATA *hallSensor, FOC_VARS_CFG *focVarsCfg);

/*!
* @brief Hall sensor PWDT interrupt service function will call this function when motor run at middle or high speed (unswap status).
*        When enter PWDT interrupt, this function calculate motor's frequency. If the frequenct lower than threshold value, will change unswap status to swaps status.
*        What's more, this function will set rotor degree limits according to hall status.
*
* @param[in] negSectorWidth: PWDT module output, negtive pulse width
* @param[in] posSectorWidth: PWDT module output, postive pulse width
* @param[in] hallSensor: HALL_SENSOR_DATA type pointer, pointer to g_hallSensor variable
* @param[in] hallSensorCfg: HALL_SENSOR_CFG type pointer, pointer to defHallSensorCfg variable, which is hall mode configuration
* @param[in] focVarsCfg: FOC_VARS_CFG type pointer, pointer to defFocVars, which configure whole FOC control
* @return none
*/
void HallSensor_UnswapState(uint16_t negSectorWidth, uint16_t posSectorWidth, HALL_SENSOR_DATA *hallSensor, HALL_SENSOR_CFG *hallSensorCfg, FOC_VARS_CFG *focVarsCfg);

/*!
* @brief After hall self-learn, this function will  transfer the install order to list value.
*        Hall install have many methods, then the hall status order will be different according to install method.
*        This function will be called after hall self-learn finished.
*
* @param[in] hallSelfLearn: HALL_LEARN_DATA type pointer, pointer to g_hallSelfLearn variable
* @param[in] focVarsCtrl: FOC_VARS_CTRL type pointer, pointer to g_focVarsCtrl, which control whole FOC algorithm
* @return none
*/
void HallSensor_SelfLearnOrderToList(HALL_LEARN_DATA *hallSelfLearn, FOC_VARS_CTRL *focVarsCtrl);

/*!
* @brief The hall install have two methods, 60 degree method hall status is  0 1 3 4 6 7, 120 degree method hall status is 1 2 3 4 5 6.
*        This function can set 60 degree install hall status transfer to 120 degree install hall status.
*        This function will be called when read PWDT hall status value.
*
* @param[in] hallSensor: HALL_SENSOR_DATA type pointer, pointer to g_hallSensor variable
* @return none
*/
void HallSensor_60ModeTo120Mode(HALL_SENSOR_DATA *hallSensor);

/*!
* @brief After hall self-learn, this function will  calculate different hall status starting point angle.
*        Hall installation can not aviod have erro, so hall self-learn must get this error, otherwise whill result to rotor position calculate incorrect.
*        This function will be called after hall self-learn finished.
*
* @param[in] selfLearnTheta: Hall self-learn result, indicate hall install error
* @return none
*/
void HallSensor_SelfLearnThetaToList(uint16_t selfLearnTheta);

/*!
* @brief Hall sensor self-learn main function, mainly contain two parts, hall installation order and err.
*        Firstly, force rotor to different position, interval is 10 degree, and get the hall change position.
*        Secondly, force rotor to different position, interval is 2 degree, narrow the range, and get the hall change position.
*
* @param[in] hallStatus: PWDT module output hall status
* @param[in] hall: HALL_SENSOR type pointer, pointer to g_hall variable
* @param[in] focVars: FOC_VARS type pointer, pointer to g_focVars, which contain FOC configure and FOC control
* @return Given rotor position
*/
uint16_t HallSensor_SelfLearn(uint8_t hallStatus, HALL_SENSOR *hall, FOC_VARS *focVars);

/*!
* @brief Hall sensor ten degree self-learn function, force rotor to different position, interval is 10 degree, and get the hall change position.
*        This function will be called in hall self-learn main function at every N*Ts.
*        If get the hall chang degree, will enter two degree self-learn stage.
*
* @param[in] hallStatus: PWDT module output hall status
* @param[in] hallSelfLearn: HALL_LEARN_DATA type pointer, pointer to g_hallSelfLearn variable
* @param[in] hallSensor: HALL_SENSOR_DATA type pointer, pointer to g_hallSensor variable
* @param[in] focVarsCtrl: FOC_VARS_CTRL type pointer, pointer to g_focVarsCtrl, which control whole FOC algorithm
* @return none
*/
void HallSensor_TenDegreeSelfLearnNTs(uint8_t hallStatus, HALL_LEARN_DATA *hallSelfLearn, HALL_SENSOR_DATA *hallSensor, FOC_VARS_CTRL *focVarsCtrl);

/*!
* @brief Hall sensor ten degree self-learn function, force rotor to different position, interval is 10 degree, and get the hall change position.
*        This function will be called in hall self-learn main function at every Ts.
*        If get the hall chang degree, will enter two degree self-learn stage.
*
* @param[in] hallSelfLearn: HALL_LEARN_DATA type pointer, pointer to g_hallSelfLearn variable
* @param[in] hallSensorCfg: HALL_SENSOR_CFG type pointer, pointer to defHallSensorCfg variable, which is hall mode configuration
* @param[in] focVarsCtrl: FOC_VARS_CTRL type pointer, pointer to g_focVarsCtrl, which control whole FOC algorithm
* @return none
*/
void HallSensor_TenDegreeSelfLearnTs(HALL_LEARN_DATA *hallSelfLearn, HALL_SENSOR_CFG *hallSensorCfg, FOC_VARS_CTRL *focVarsCtrl);

/*!
* @brief Hall sensor two degree self-learn function, force rotor to different position, interval is two degree, and get the hall change position.
*        This function will be called in hall self-learn main function at every N*Ts.
*        If get the hall chang degree, the whole hall self-learn is finished.
*
* @param[in] hallStatus: PWDT module output hall status
* @param[in] hallSelfLearn: HALL_LEARN_DATA type pointer, pointer to g_hallSelfLearn variable
* @param[in] hallSensor: HALL_SENSOR_DATA type pointer, pointer to g_hallSensor variable
* @param[in] focVarsCtrl: FOC_VARS_CTRL type pointer, pointer to g_focVarsCtrl, which control whole FOC algorithm
* @return none
*/
void HallSensor_TwoDegreeSelfLearnNTs(uint8_t hallStatus, HALL_LEARN_DATA *hallSelfLearn, HALL_SENSOR_DATA *hallSensor, FOC_VARS_CTRL *focVarsCtrl);

/*!
* @brief Hall sensor two degree self-learn function, force rotor to different position, interval is two degree, and get the hall change position.
*        This function will be called in hall self-learn main function at every Ts.
*        If get the hall chang degree, the whole hall self-learn is finished.
*
* @param[in] hallSelfLearn: HALL_LEARN_DATA type pointer, pointer to g_hallSelfLearn variable
* @param[in] hallSensorCfg: HALL_SENSOR_CFG type pointer, pointer to defHallSensorCfg variable, which is hall mode configuration
* @param[in] focVarsCtrl: FOC_VARS_CTRL type pointer, pointer to g_focVarsCtrl, which control whole FOC algorithm
* @return none
*/
void HallSensor_TwoDegreeSelfLearnTs(HALL_LEARN_DATA *hallSelfLearn, HALL_SENSOR_CFG *hallSensorCfg, FOC_VARS_CTRL *focVarsCtrl);

/*!
* @brief When motor run at low speed (swap status), the ASR output iqRef will be very samll, and the torque will be small.
*        To make sure motor start up smoothly, this function boost trque at low speed properly.
*
* @param[in] hallSensor: HALL_SENSOR_DATA type pointer, pointer to g_hallSensor variable
* @param[in] hallSensorCfg: HALL_SENSOR_CFG type pointer, pointer to defHallSensorCfg variable, which is hall mode configuration
* @param[in] pidParam: PID_CFG type pointer, pointer to g_pidParam variable, which is iq refence max and min limit
* @param[in] focVarsCtrl: FOC_VARS_CTRL type pointer, pointer to g_focVarsCtrl, which control whole FOC algorithm
* @return none
*/
void HallSensor_LowSpdBoostTrq(HALL_SENSOR_DATA *hallSensor, HALL_SENSOR_CFG *hallSensorCfg, PID_CFG *pidParam, FOC_VARS_CTRL *focVarsCtrl);

/*!
* @brief Motor run direction command is given by outside, but motor actual run direction maybe different form command.
*        If the motor actual run direction is different from command, the rotor position calculate will be disorder.
*
* @param[in] hallSensor: HALL_SENSOR_DATA type pointer, pointer to g_hallSensor variable
* @param[in] hallSensorCfg: HALL_SENSOR_CFG type pointer, pointer to defHallSensorCfg variable, which is hall mode configuration
* @return none
*/
void HallSensor_JudgeRotorDir(HALL_SENSOR_DATA *hallSensor, HALL_SENSOR_CFG *hallSensorCfg);

/*!
* @brief This function is called by HallSensor_HallWidthLearn, in hall width learn process, the first step is forcing rotor position to zero.
*        This function through give idRef and eletric angle force rotor to zero position.
*
* @param[in] hallSelfLearn: HALL_LEARN_DATA type pointer, pointer to g_hallSelfLearn variable
* @param[in] focVars: FOC_VARS type pointer, pointer to g_focVars, which contain FOC configure and FOC control
* @return none
*/
void HallSensor_SetZeroPos(HALL_LEARN_DATA *hallSelfLearn, FOC_VARS *focVars);

/*!
* @brief Hall sensor have installation error, result to six sector width are different.
*        If leave out the sector width difference, the rotor position calculate will have error.
*        This function mainly self-learn six sector width in order to improve estimation accuracy.
*
* @param[in] hallSelfLearn: HALL_LEARN_DATA type pointer, pointer to g_hallSelfLearn variable
* @param[in] focVars: FOC_VARS type pointer, pointer to g_focVars, which contain FOC configure and FOC control
* @return none
*/
void HallSensor_HallWidthLearn(HALL_LEARN_DATA *hallSelfLearn, FOC_VARS *focVars);

/*!
* @brief PWDT module interrput service function for hall width self-learn, this function is called by PWDT Isr function.
*        Save the interval value between two pwdt isr, then calculate the sector width.
*
* @param[in] hallStatus: PWDT module output hall status
* @param[in] negSectorWidth: PWDT module output, negtive pulse width
* @param[in] posSectorWidth: PWDT module output, postive pulse width
* @param[in] hallSelfLearn: HALL_LEARN_DATA type pointer, pointer to g_hallSelfLearn variable
* @param[in] hallSensor: HALL_SENSOR_DATA type pointer, pointer to g_hallSensor variable
* @return none
*/
void HallSensor_SelfLearnPwdt(uint8_t hallStatus, uint16_t negSectorWidth, uint16_t posSectorWidth, HALL_LEARN_DATA *hallSelfLearn, HALL_SENSOR_DATA *hallSensor);

/*!
* @brief Provide electric angle interface. This function will be called in FOC isr function.
*
* @param[in] hall: HALL_SENSOR type pointer, pointer to g_hall variable, which include hall_sensor, hall_cfg, hall_learn
* @return none
*/
uint16_t Get_HALL_ElecAngle(HALL_SENSOR *hall);

/* ======================================  Functions define  ======================================== */

#endif /* __HALL_SENSOR_H__ */

/* =============================================  EOF  ============================================== */
