/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __PULSE_INJECT_H__
#define __PULSE_INJECT_H__
/*!
* @file pulse_inject.h
*
* @brief This file provides motor rotor initial position checking functions interfaces.The principle is saturation characteristic of inductance.
*
*/

/* ===========================================  Includes  =========================================== */
#include <stdint.h>

/* ============================================  Define  ============================================ */
#define BUS_CUR_DATA_THRES      1       /*!< ad data threshold */
#define MAX_AD_SAMPLE_VAL       4050    /*!< if ad sample value bigger than this value, then the ad value is incorrect */
#define SECTOR_60_DEGREE        10923   /*!< 60 degree in Q16 format */

/* ===========================================  Typedef  ============================================ */
/*!
* @brief Pulse inject to get initial position function status
*/
typedef enum {                          /*!< Define check initial position step */
    CHIP_CONFIG = 0,                    /*!< Status 1: config */
    SELF_LEARNING,                      /*!< Status 2: self-learning */
    PULSE_INJECTING,                    /*!< Status 3: pulse injecting */
    CHIP_CONFIG_RESET,                  /*!< Status 4: recover config */
    PULSE_INJECTION_ERR,                /*!< Status 5: error */
} INIT_POS_STATUS;

/*!
* @brief Pulse inject function configuration
*/
typedef struct
{
    uint16_t selfLearnThres;            /*!< Pulse inject self learn current threshold */
    uint16_t selfLearnDutyStep;         /*!< Pulse inject self learn pwm duty increase step */
} PULSE_INJ_CFG;

/*!
* @brief Pulse inject data
*/
typedef struct
{
    uint8_t pulseInjDoneFlag;           /*!< 0:pulse injection unfinished.1:pulse injection finished */
    uint8_t pulseInjFlag;               /*!< PULSE_INJECTING status finished flag */
    uint8_t flowState;                  /*!< Check initial position,initial sector transfer to openkick */
    uint8_t selfLearnFlag;              /*!< 0:Self-learn unfinished 1:Self-learn finished */
    uint8_t subStatus;                  /*!< Sub-program status */
    uint8_t periodCnt;                  /*!< Check initial position ad interrupt counter */
    uint16_t busCur[6];                 /*!< AD data array */
    uint16_t busCurVal;                 /*!< Bus current. */
    uint16_t wait;                      /*!< Delay counter */
    uint16_t pwmTs;                     /*!< PWM period value */
    uint16_t section;                   /*!< Pulse section */
    uint16_t curFirst;                  /*!< Self-learn bus current first val */
    uint16_t initEletricTheta;          /*!< Initial rotor position  */
    uint16_t pulseInjErrCnt;            /*!< Pulse injection error counter */
} PULSE_INJECT_DATA;

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Initial position pulse inject contain two parts, pulse width self-learn and pulse injection.
*        This function can get proper pulse injection width, which is different with each motor.
*        Cann't call PulseInj_SetChlVal, otherwiese enter ad isr will call this function immediately again,
*        and will result to first fulse output err.
*
* @param[in] pulseInj: PULSE_INJECT_DATA type pointer, pointer to g_pulseInject variable
* @return none
*/
void PulseInject_SelfLearn(PULSE_INJECT_DATA *pulseInj);

/*!
* @brief After pulse injection, get six direction bus current, base on saturation characteristic of inductance, calculate rotor initial position
*        Initial position pulse inject data handle, output initial position sector.
*        This function handle data only need bus current, so initial position check function is also fit to bldc control.
*
* @param[in] pulseInj: PULSE_INJECT_DATA type pointer, pointer to g_pulseInject variable
* @return none
*/
void PulseInject_DataHandle(PULSE_INJECT_DATA *pulseInj);

/*!
* @brief Initial position pulse inject ADC interrupt function, it mainly contain two parts, pulse width self-learn and pulse injection.
*        This function is only for initial position check function. The interrupt address will recover after get initial position.
*
* @param[in] pulseInj: PULSE_INJECT_DATA type pointer, pointer to g_pulseInject variable
* @param[in] pulseInjCfg: PULSE_INJ_CFG type pointer, pointer to g_pulseInjCfg variable, which is initial postion check configuration
* @return none
*/
void InitPos_AdcSubCallBack(PULSE_INJECT_DATA *pulseInj, PULSE_INJ_CFG *pulseInjCfg);

/*!
* @brief Output initial position check function rotor position
*
* @param[in] pulseInj: PULSE_INJECT_DATA type pointer, pointer to g_pulseInject variable
* @return none
*/
uint16_t InitPos_GetPosAngle(PULSE_INJECT_DATA *pulseInj);

/* ======================================  Functions define  ======================================== */

#endif /* __INIT_POS_H__ */

/* =============================================  EOF  ============================================== */
