/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef _FOC_KEYS_H
#define _FOC_KEYS_H
/*!
* @file foc_keys.h
*
* @brief This file provides keys control functions interface.
*
*/

/* ===========================================  Includes  =========================================== */
#include <stdint.h>
#include "ac780x_timer_reg.h"

/* ============================================  Define  ============================================ */
#define MOTORAPP_MAIN_VER           1                   /*!< Main version of MotorApp code project */
#define MOTORAPP_SUB_VER            2                   /*!< Sub-version of MotorApp code project */
#define MOTORAPP_REV_VER            3                  /*!< Rev-version of MotorApp code project */

#define MOTORAPP_VER_YEAR           2022                /*!< Year of the current MotorApp code version released */
#define MOTORAPP_VER_MONTH          2                   /*!< Month of the current MotorApp code version released */
#define MOTORAPP_VER_DAY            17                  /*!< Date of the current MotorApp code version released */

#define MASTER                      0                   /*!< Master */
#define MCU_TRUNK_AC780X            1                   /*!< MCU motor code demo branch */
#define MCU_TRUNK_AC780X_FAN        2                   /*!< MCU motor code fan branch */
#define MCU_TRUNK_AC780X_HALL       3                   /*!< MCU motor code hall branch */
#define MCU_TRUNK_AC780X_1SHUNT     4                   /*!< MCU motor code small board branch */
#define MCU_TRUNK_AC780X_HV         5                   /*!< MCU motor code high voltage branch */

#define MOTORAPP_BRANCH             MCU_TRUNK_AC780X_1SHUNT    /*!< Code branch */

/* define Key AD Value range */
#define START_STOP_MIN              0                   /*!< Lower limit of AD value corresponding to Start-stop key */
#define START_STOP_MAX              588                 /*!< Upper limit of AD value corresponding to Start-stop key */
#define FORWARD_REVERSE_MIN         589                 /*!< Lower limit of AD value corresponding to Forward-reverse key */
#define FORWARD_REVERSE_MAX         901                 /*!< Upper limit of AD value corresponding to Forward-reverse key */
#define FAST_MIN                    1368                /*!< Lower limit of AD value corresponding to Fast key */
#define FAST_MAX                    2080                /*!< Upper limit of AD value corresponding to Fast key */
#define SLOW_MIN                    2089                /*!< Lower limit of AD value corresponding to Slow key */
#define SLOW_MAX                    3162                /*!< Upper limit of AD value corresponding to Slow key */
#define SUSPEND_MIN                 3500                /*!< Lower limit of AD value when no key is pressed */
#define SUSPEND_MAX                 4095                /*!< Upper limit of AD value when no key is pressed */

#define CONFIRM_COUNT               500                 /*!< Count value used to confirm the key is pressed */

/* ===========================================  Typedef  ============================================ */
/*!
* @brief Define motor control status enumeration.
*/
typedef enum
{
    POWER_ON            =   0,          /*!< Power on, enter this state after the control board is powered on */
    IDLE                =   1,          /*!< Idle( stand by), enter this state after POWER_ON and the parameter initialization is finished */
    START               =   2,          /*!< Start, if not in TAIL_HEAD_WIND mode, enter this state after the Start key is pressed */
    ANGLE_JUDGE         =   3,          /*!< Angle judge, enter this state after Start key is pressed, use PULSE_INJECTION function to get motor initial position */
    SPEED_JUDGE         =   4,          /*!< Speed judge, enter this state after Start key is pressed in TAIL_HEAD_WIND mode */
    FLY_RUN             =   5,          /*!< Fly run, enter this state after SPEED_JUDGE is complete in TAIL_HEAD_WIND mode */
    START_BRAKE         =   6,          /*!< Start brake, enter this state when the brake function is required. */
    ALIGN               =   7,          /*!< Align, enter this state when the initial angle positioning needs to be performed */
    RUN                 =   8,          /*!< Run, the control system enters this state and performs motor drive control */
    STOP                =   9,          /*!< Stop, the control system stops driving the motor and enters this state */
    SLOW_DOWN           =   10,         /*!< Slow down, this state is entered when the deceleration stop mode(Slow down) is selected and the stop command is executed */
    STOP_BRAKE          =   11,         /*!< Stop brake, enter this state when the brake function is required. */
    IDENTIFY            =   12,         /*!< Identify, enter this state and start motor parameters identification */
    FAULT               =   13,         /*!< Fault, this state is entered when the control system is running fault */
    RESTART             =   14,         /*!< Restart, this state is entered when the startup error happens */
    FLASH_TASK          =   15,         /*!< Flash task, this state is entered when master commands flash operation */
    HALL_SELFLEARN      =   16,         /*!< Hall sensor self learn, this state is entered when needed to obtain Hall sensor installation information */
} MC_STATUS;

/* ==========================================  Variables  =========================================== */
extern MC_STATUS g_mcStatus;            /*!< Define motor operation status */

static uint8_t s_keyStartPressed = 0;   /*!< Used to judge whether the Start/Stop key acts */
static uint8_t s_keyDirPressed = 0;     /*!< Used to judge whether the Forward/Reverse key acts */
static uint8_t s_keyUpPressed = 0;      /*!< Used to judge whether the Fast key acts */
static uint8_t s_keyDownPressed = 0;    /*!< Used to judge whether the Slow key acts */
static uint16_t s_keyCmdCount = 0;      /*!< Keys action detection counter */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Get the response of pressing each key on the control board.
*        The keys action are judged by the ADC value:
*        when ADC value is in [ 0, START_STOP_MAX ], it corresponds to the Start / Stop key;
*        when ADC value is in [ FORWARD_REVERSE_MIN, FORWARD_REVERSE_MAX ], it corresponds to the Forward / Reverse key;
*        when ADC value is in [ FAST_MIN, FAST_MAX ], it corresponds to the Fast key;
*        when ADC value is in [ SLOW_MIN, SLOW_MAX ], it corresponds to the Slow key.
*
* @param[in] none
* @return none
*/
void MC_Keys_Read(void);

/* ======================================  Functions define  ======================================== */

#endif /* _FOC_KEYS_H */

/* =============================================  EOF  ============================================== */
